package de.ozgcloud.apilib.user;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.user.grpc.userprofile.UserProfileServiceGrpc.UserProfileServiceBlockingStub;
import de.ozgcloud.user.userprofile.GrpcGetUserProfileRequest;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@RequiredArgsConstructor
public class GrpcOzgCloudUserProfileService implements OzgCloudUserProfileService {

	@GrpcClient("user-manager")
	private final UserProfileServiceBlockingStub serviceStub;

	private final UserProfileMapper mapper;

	private final OzgCloudCallContextProvider contextProvider;

	@Override
	public OzgCloudUserProfile getById(OzgCloudUserId userId) {
		var grpcUserProfileResponse = getUserProfileServiceStub().getById(buildGetUserRequest(userId));

		return mapper.mapFromGrpc(grpcUserProfileResponse.getUserProfile());
	}

	private GrpcGetUserProfileRequest buildGetUserRequest(OzgCloudUserId id) {
		return GrpcGetUserProfileRequest.newBuilder().setUserId(id.toString())
				.build();
	}

	private UserProfileServiceBlockingStub getUserProfileServiceStub() {
		return serviceStub.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}
}
